'''
This module implements the Hidden Markov Model shown in the text, Figure 15.2, and 
demonstrates the filtering (FORWARD) algorithm.

This example is based on Michael Hamilton's HMM code:
    see http://www.cs.colostate.edu/~hamiltom/code.html 
Note that Hamilton's code has an oddity the prevents it from working properly
on problems with assymetric transition models. Thus, the text example (shown here) 
works, but Thrun's happy/grumpy example on-line does not.

@author: kvlinden
@version Jan 5, 2013
'''

from hamilton.hmm import HMM, forward, backward, viterbi, baum_welch
import numpy

'''
This implements the text's umbrella example, Figure 15.2 and Section 15.2.1, for times 0:2.

transition_model - specifies the (symmetric) hidden state transition probabilities, as in:

           Rain   -Rain
     Rain  0.7     0.3
    -Rain  0.3     0.7

sensor_model - specifies the probability distribution for the umbrella, as in:

           Umbrella  -Umbrella
     Rain   0.9        0.1
    -Rain   0.2        0.8
    
symbols - lists the possible values for the observable random variable

observations - lists the observed values for the observable variable over time 

The code assumes a uniform distribution of the prior probabilities (as does the text).

'''  
transition_model = numpy.array([ [ 0.7, 0.3 ], \
                                 [ 0.3, 0.7 ] ])
sensor_model = numpy.array([ [ 0.9, 0.1 ], \
                             [ 0.2, 0.8 ] ])
symbols = ['U', '-U']
observations = ['U', 'U', 'U']

hmm = HMM(2, A=transition_model, B=sensor_model, V=symbols)

# This uses the forward filtering algorithm (equation 15.5) to compute P(R1|u1), P(R2|u1,u2) 
# and P(R3|u1,u2,u3) (see Section 15.2.1 for the details of the first two values).
print 'Forward filter values:\n' + str(forward(hmm, observations)[1])     

